import logging
from hashlib import new

from flask_login import current_user

from app import db
from app.auth.models import User
from flask import abort, current_app, redirect, render_template, url_for
from flask.helpers import flash
from .forms import UserAdminForm, UserForm, UserUpdateForm


class UserController():

    def __init__(self):
        self.logger = logging.getLogger(__name__)

    def list_users(self):
        users = User.get_all()
        form = UserAdminForm()
        return render_template("users.html", users=users, form=form)

    def user_form(self):
        form = UserForm()
        error = None
        if form.validate_on_submit():
            name = form.name.data
            email = form.email.data
            password = form.password.data
            is_admin=form.is_admin.data
            # Comprobamos que no hay ya un usuario con ese email
            user = User.get_by_email(email)
            if user is not None:
                error = f'El email {email} ya está siendo utilizado por otro usuario'
            else:
                # Creamos el usuario y lo guardamos
                user = User(name=name, email=email, is_admin=is_admin)
                user.set_password(password)
                user.save()
                self.logger.info(f'Guardando el usuario {user.id}')
                return redirect(url_for('admin.list_users'))
        return render_template("user_form.html", form=form, error=error)


    def update_user_form(self, user_id):
        user = User.get_by_id(user_id)
        error = None
        cambio_mail = False
        if user is None:
            self.logger.info(f'El usuario {user_id} no existe')
            abort(404)
        # Crea un formulario inicializando los campos con
        # los valores del usuario.
        form = UserUpdateForm(obj=user)
        if form.validate_on_submit():
            # Actualiza los campos del usuario existente
            user.name = form.name.data
            if user.email != form.email.data:
                cambio_mail = True
            user.is_admin = form.is_admin.data
            # Comprobamos que no hay ya un usuario con ese email
            user_mail = User.get_by_email(form.email.data)
            if (cambio_mail is True and user_mail is not None ):
                flash(f'El email {user_mail.email} ya está siendo utilizado por otro usuario')
            else:
                user.email = form.email.data
                user.save()
                self.logger.info(f'Guardando el usuario {user_id}')
                return redirect(url_for('admin.list_users'))
        return render_template("user_update_form.html", form=form, user=user, error=error)

    def delete_user(self, user_id):
        self.logger.info(f'Se va a eliminar al usuario {user_id}')
        user = User.get_by_id(user_id)
        if user is None:
            self.logger.info(f'El usuario {user_id} no existe')
            abort(404)
        user.delete()
        self.logger.info(f'El usuario {user_id} ha sido eliminado')
        return redirect(url_for('admin.list_users'))

userController=UserController()
