from flask_wtf import FlaskForm
from wtforms.validators import DataRequired, Length, Email, EqualTo
from wtforms import (StringField, SubmitField, PasswordField,  BooleanField)

class UserAdminForm(FlaskForm):
    is_admin = BooleanField('Administrador')
    submit = SubmitField('Guardar')

class UserForm(FlaskForm):
    name = StringField('Nombres del usuario', validators=[DataRequired(message="Ingrese los nombres del usuario"), Length(min=5, max=64, message="El nombre del usuario debe tener entre 5 y 64 caracteres")])
    email = StringField('Email', validators=[DataRequired(message="Ingrese un correo electrónico"), Email("Ingrese un correo electrónico válido"), Length(min=5, message="El correo electrónico debe tener al memos 5")])
    password = PasswordField('Clave de acceso', validators=[DataRequired(message="Ingrese la clave de acceso del usuario"), Length(min=8, message="La clave de acceso debe tener al menos 8 caracteres"), EqualTo('password_confirm', message='La clave y su confirmación no coinciden')])
    password_confirm = PasswordField('Cofirmación de la clave de acceso', validators=[DataRequired(message="Ingrese la cofirmación de la clave de acceso"), Length(min=8)])
    is_admin = BooleanField('Es administrador?')
    submit = SubmitField('Crear')

class UserUpdateForm(FlaskForm):
    name = StringField('Nombres del usuario', validators=[DataRequired(message="Ingrese los nombres del usuario"), Length(min=5, max=64, message="El nombre del usuario debe tener entre 5 y 64 caracteres")])
    email = StringField('Email', validators=[DataRequired(message="Ingrese un correo electrónico"), Email("Ingrese un correo electrónico válido"), Length(min=5, message="El correo electrónico debe tener al memos 5")])
    is_admin = BooleanField('Es administrador?')
    submit = SubmitField('Modificar')
    