from flask_wtf import FlaskForm
from wtforms import StringField, SubmitField, PasswordField, BooleanField
from wtforms.validators import DataRequired, Email, Length, EqualTo



class LoginForm(FlaskForm):
    email = StringField('Email', validators=[DataRequired(message="Ingrese el correo electrónico del usuario")])
    password = PasswordField('Password', validators=[DataRequired(message="Ingrese la contraseña")])
    remember_me = BooleanField('Recuérdame')
    submit = SubmitField('Login')

class SignupForm(FlaskForm):
    name = StringField('Nombres del usuario', validators=[DataRequired(message="Ingrese los nombres del usuario"), Length(min=5, max=64, message="El nombre del usuario debe tener entre 5 y 64 caracteres")])
    email = StringField('Email', validators=[DataRequired(message="Ingrese un correo electrónico"), Email("Ingrese un correo electrónico válido"), Length(min=5, message="El correo electrónico debe tener al memos 5")])
    password = PasswordField('Clave de acceso', validators=[DataRequired(message="Ingrese la clave de acceso del usuario"), Length(min=8, message="La clave de acceso debe tener al menos 8 caracteres"), EqualTo('password_confirm', message='La clave y su confirmación no coinciden')])
    password_confirm = PasswordField('Cofirmación de la clave de acceso', validators=[DataRequired(message="Ingrese la cofirmación de la clave de acceso"), Length(min=8)])
    submit = SubmitField('Registrar')