from hashlib import new
import os
import logging
from flask import render_template, redirect, url_for, abort, current_app
from flask.helpers import flash
from flask_login import login_required, current_user
from app.models import Documento, ModeloNer, Proyecto
#from . import registered_bp
from .forms import CreateDocumentsForm, DcoumentoDeleteForm, ProyectoForm, UserAdminForm, ProyectoDeleteForm
#from app.auth.decorators import admin_required
from app.auth.models import User
from app.utils.actor_recognition import ActorRecognition
from app.utils.extract_relations import ExtracRelations
from app.utils.generate_graph import GenerateGraph
from app.utils.extract_text import extract_text_from_pdf
from app.utils.clean_text import clean_text
from app.utils.actor_relations import ActorRelations
from werkzeug.utils import secure_filename
from app import db


class ProyectoController():

    def __init__(self):
        self.logger = logging.getLogger(__name__)

    def index():
        return render_template("registered/index.html")

    def list_proyectos(self):
        proyectos = db.session.query(Proyecto).filter_by(user_id=current_user.id)
        form = ProyectoDeleteForm()
        return render_template("registered/proyectos.html", proyectos=proyectos, form=form)

    def proyecto_form(self, proyecto_id=None):
        form = ProyectoForm()
        modelos_ner = [(c.id, c.nombre) for c in ModeloNer.query.all()]
        form.modelo_ner.choices = modelos_ner
        if form.validate_on_submit():
            proyecto = Proyecto(nombre = form.nombre.data,
                                descripcion = form.descripcion.data,
                                distancia_palabras = form.distancia_palabras.data,
                                user_id=current_user.id, 
                                modelo_ner_id = form.modelo_ner.data)
            proyecto.save()
            return redirect(url_for('registered.list_proyectos'))
        return render_template("registered/proyecto_form.html", form=form)

    def show_proyecto(self, proyecto_id):
        proyecto = Proyecto.get_by_id(proyecto_id)
        
        if proyecto is None:
            self.logger.info(f'El proyecto {proyecto_id} no existe')
            abort(404)
        
        form = DcoumentoDeleteForm()
        return render_template("registered/proyecto.html", proyecto=proyecto, form=form)

    def update_proyecto_form(self, proyecto_id):
        proyecto = Proyecto.get_by_id(proyecto_id)
        
        if proyecto is None:
            self.logger.info(f'El post {proyecto_id} no existe')
            abort(404)
        # Crea un formulario inicializando los campos con
        # los valores del proyecto.
        form = ProyectoForm(obj=proyecto)
        modelos_ner = [(c.id, c.nombre) for c in ModeloNer.query.all()]
        form.modelo_ner.choices = modelos_ner
        if form.validate_on_submit():
            # Actualiza los campos del proyecto existente
            proyecto.nombre = form.nombre.data,
            proyecto.descripcion = form.descripcion.data,
            proyecto.distancia_palabras = form.distancia_palabras.data,
            proyecto.user_id=current_user.id, 
            proyecto.modelo_ner_id = form.modelo_ner.data
            proyecto.save()
            self.logger.info(f'Guardando el proyecto {proyecto_id}')
            return redirect(url_for('registered.list_proyectos'))
        return render_template("registered/proyecto_form.html", form=form, proyecto=proyecto)


    def delete_proyecto(self, proyecto_id):
        self.logger.info(f'Se va a eliminar el proyecto {proyecto_id}')
        post = Proyecto.get_by_id(proyecto_id)
        if post is None:
            self.logger.info(f'El proyecto {proyecto_id} no existe')
            abort(404)
        post.delete()
        self.logger.info(f'El proyecto {proyecto_id} ha sido eliminado')
        return redirect(url_for('registered.list_proyectos'))


    def create_document(self, proyecto_id):
        proyecto = Proyecto.get_by_id(proyecto_id)
        
        if proyecto is None:
            self.logger.info(f'El proyecto {proyecto_id} no existe')
            abort(404)
        
        form = CreateDocumentsForm()

        if form.validate_on_submit():
            for file in form.documentos.data:
                documento = Documento()
                documento.nombre = file.filename
                file_filename = secure_filename(file.filename)
                documents_dir = current_app.config['PROYECTOS_DOCUMENTS_DIR'] + os.path.sep + str(proyecto_id) + os.path.sep
                os.makedirs(documents_dir, exist_ok=True)
                file_path = os.path.join(documents_dir, file_filename)
                documento.ubicacion = file_path
                documento.proyecto=proyecto
                file.save(file_path)
                db.session.add(documento)
            
            db.session.commit()
            
            return redirect(url_for('registered.show_proyecto', proyecto_id=proyecto_id))


        return render_template("registered/create_document.html", proyecto=proyecto, form=form)

    def delete_documento(self, proyecto_id, documento_id):
        self.logger.info(f'Se va a eliminar el documento {documento_id}')
        documento = Documento.get_by_id(documento_id)
        if documento is None:
            self.logger.info(f'El documento {documento_id} no existe')
            abort(404)
        
        documento.delete()
        if os.path.isfile(documento.ubicacion):
            os.remove(documento.ubicacion)
        self.logger.info(f'El proyecto {documento_id} ha sido eliminado')

        return redirect(url_for('registered.show_proyecto', proyecto_id=proyecto_id))

    def identify_actors(self, proyecto_id):
        self.logger.info(f'Se van a identificar actores del proyecto {proyecto_id}')
        proyecto = Proyecto.get_by_id(proyecto_id)
        if proyecto is None:
            self.logger.info(f'El proyecto {proyecto_id} no existe')
            abort(404)
        
        if len(proyecto.documentos.all()) == 0 :
            flash('El proyecto debe tener al menos un documento cargado para poder generar el mapa')
            return redirect(url_for('registered.show_proyecto', proyecto_id=proyecto_id))
        

        dir_model = current_app.config['MODELOS_DOCUMENTS_DIR'] + os.path.sep + 'modelo-tok2vec'
        """Se debe consultar el modelo en la base de datos"""
        
        graph = GenerateGraph()

        files_path= current_app.config['PROYECTOS_DOCUMENTS_DIR'] + os.path.sep + str(proyecto.id)
        files = os.listdir(files_path) 
        files = [f for f in os.listdir(files_path) if os.path.isfile(os.path.join(files_path, f))]

        #print(files)

        #logger.info(f'Archivos:  {files}')
        todos = ''
        relations=[]
        for file in files:
            #print(os.path.join(files_path, file))
            text_pdf = extract_text_from_pdf(os.path.join(files_path, file))
            #print(text_pdf)
            text_pdf = clean_text(text_pdf)
            actors_list = ActorRecognition(text_pdf, dir_model).getActors()
            extract_relations = ExtracRelations(text_pdf, actors_list)
            file_relations = extract_relations.getRelationships(proyecto.distancia_palabras)
            relations = list(set().union(relations,file_relations))
            #tomar relations y crear lista de relaciones con nombres del actor
            #para esto se deben buscar en la tabla actores los nombres que vienen en la relación
            #y si no existen se busca si aparece como sinonimo
            #acá se puede crear un diccionario en python con los actores y sus sinonimos
            # actor = actor + actor=sinonimo, y así ahorrar busquedas en la BD
        
        actor_rel = ActorRelations()
        relations_out_sinonimos = actor_rel.replaceNamesRelations(relations)

        #Encontrados
        #print('Relaciones sin sinonimos')
        #print(relations_out_sinonimos)
        graph.generateDBGraph(relations_out_sinonimos)
            
        #    if entidades > 0:
        #        print([(ent.text, ent.label_) for ent in entidades])

        #text = sentence.replace(',', ' ,')
        #text = " ".join(text.split())
        #logger.info(f'Oración:  {sentence}')
        
        #actors_list = ActorRecognition(sentence, dir_model).getActors()
        #print('Listado de actores: ')
        #print(actors_list)
        #actors_text = ', '.join([str(item) for item in actors_list])
        #extract_relations = ExtracRelations(sentence, actors_list)
        #relations = extract_relations.getRelationships()
        #print('Realciones: ')
        #print(relations)
        
        #graph.generateDBGraph(relations)

        return render_template("registered/show_actors.html", proyecto=proyecto, actors=todos)



proyectoController=ProyectoController()